import sys
import os
import json
import re

if sys.version_info[0] >= 3:
    unicode = str

def is_bool(x, val=None):
    return isinstance(x, bool) and (val is None or x == val)

def is_dict(x):
    return isinstance(x, dict)

def is_list(x):
    return isinstance(x, list)

def is_int(x, val=None):
    return (isinstance(x, int) or isinstance(x, long)) and (val is None or x == val)

def is_string(x, val=None):
    return (isinstance(x, str) or isinstance(x, unicode)) and (val is None or x == val)

def matches(s, pattern):
    return is_string(s) and bool(re.search(pattern, s))

def check_list_match(match, actual, expected, check=None, check_exception=None, missing_exception=None, extra_exception=None, allow_extra=False):
    """
    Handle the common pattern of making sure every actual item "matches" some
    item in the expected list, and that neither list has extra items after
    matching is completed.

    @param match: Callback to check if an actual item matches an expected
    item. Return True if the item matches, return False if the item doesn't
    match.
    @param actual: List of actual items to search.
    @param expected: List of expected items to match.
    @param check: Optional function to check that the actual item is valid by
    comparing it to the expected item.
    @param check_exception: Optional function that returns an argument to
    append to any exception thrown by the check function.
    @param missing_exception: Optional function that returns an argument to
    append to the exception thrown when an item is not found.
    @param extra_exception: Optional function that returns an argument to
    append to the exception thrown when an extra item is found.
    @param allow_extra: Optional parameter allowing there to be extra actual
    items after all the expected items have been found.
    """
    assert is_list(actual)
    _actual = actual[:]
    for expected_item in expected:
        found = False
        for i, actual_item in enumerate(_actual):
            if match(actual_item, expected_item):
                if check:
                    try:
                        check(actual_item, expected_item)
                    except BaseException as e:
                        if check_exception:
                            e.args += (check_exception(actual_item, expected_item),)
                        raise
                found = True
                del _actual[i]
                break
        if missing_exception:
            assert found, missing_exception(expected_item)
        else:
            assert found
    if not allow_extra:
        if extra_exception:
            assert len(_actual) == 0, [extra_exception(a) for a in _actual]
        else:
            assert len(_actual) == 0

def filter_list(f, l):
    if l is not None:
        l = list(filter(f, l))
    if l == []:
        l = None
    return l

def check_cmake(cmake):
    assert is_dict(cmake)
    assert sorted(cmake.keys()) == ["generator", "paths", "version"]
    check_cmake_version(cmake["version"])
    check_cmake_paths(cmake["paths"])
    check_cmake_generator(cmake["generator"])

def check_cmake_version(v):
    assert is_dict(v)
    assert sorted(v.keys()) == ["isDirty", "major", "minor", "patch", "string", "suffix"]
    assert is_string(v["string"])
    assert is_int(v["major"])
    assert is_int(v["minor"])
    assert is_int(v["patch"])
    assert is_string(v["suffix"])
    assert is_bool(v["isDirty"])

def check_cmake_paths(v):
    assert is_dict(v)
    assert sorted(v.keys()) == ["cmake", "cpack", "ctest", "root"]
    assert is_string(v["cmake"])
    assert is_string(v["cpack"])
    assert is_string(v["ctest"])
    assert is_string(v["root"])

def check_cmake_generator(g):
    assert is_dict(g)
    name = g.get("name", None)
    assert is_string(name)
    if name.startswith("Visual Studio"):
        assert sorted(g.keys()) == ["multiConfig", "name", "platform"]
        assert is_string(g["platform"])
    else:
        assert sorted(g.keys()) == ["multiConfig", "name"]
    assert is_bool(g["multiConfig"], matches(name, "^(Visual Studio |Xcode$|Ninja Multi-Config$)"))

def check_index_object(indexEntry, kind, major, minor, check):
    assert is_dict(indexEntry)
    assert sorted(indexEntry.keys()) == ["jsonFile", "kind", "version"]
    assert is_string(indexEntry["kind"])
    assert indexEntry["kind"] == kind
    assert is_dict(indexEntry["version"])
    assert sorted(indexEntry["version"].keys()) == ["major", "minor"]
    assert indexEntry["version"]["major"] == major
    assert indexEntry["version"]["minor"] == minor
    assert is_string(indexEntry["jsonFile"])
    filepath = os.path.join(reply_dir, indexEntry["jsonFile"])
    with open(filepath) as f:
        object = json.load(f)
    assert is_dict(object)
    assert "kind" in object
    assert is_string(object["kind"])
    assert object["kind"] == kind
    assert "version" in object
    assert is_dict(object["version"])
    assert sorted(object["version"].keys()) == ["major", "minor"]
    assert object["version"]["major"] == major
    assert object["version"]["minor"] == minor
    if check:
        check(object)

def check_index__test(indexEntry, major, minor):
    def check(object):
        assert sorted(object.keys()) == ["kind", "version"]
    check_index_object(indexEntry, "__test", major, minor, check)

def check_error(value, error):
    assert is_dict(value)
    assert sorted(value.keys()) == ["error"]
    assert is_string(value["error"])
    assert value["error"] == error

def check_error_re(value, error):
    assert is_dict(value)
    assert sorted(value.keys()) == ["error"]
    assert is_string(value["error"])
    assert re.search(error, value["error"])

reply_index = sys.argv[1]
reply_dir = os.path.dirname(reply_index)

with open(reply_index) as f:
    index = json.load(f)



from check_index import *

def check_objects(o):
    assert is_list(o)
    assert len(o) == 1
    check_index_object(o[0], "cmakeFiles", 1, 1, check_object_cmakeFiles)

def check_input(actual, expected):
    assert is_dict(actual)
    expected_keys = ["path"]

    if expected["isGenerated"] is not None:
        expected_keys.append("isGenerated")
        assert is_bool(actual["isGenerated"], expected["isGenerated"])

    if expected["isExternal"] is not None:
        expected_keys.append("isExternal")
        assert is_bool(actual["isExternal"], expected["isExternal"])

    if expected["isCMake"] is not None:
        expected_keys.append("isCMake")
        assert is_bool(actual["isCMake"], expected["isCMake"])

    assert sorted(actual.keys()) == sorted(expected_keys)

def check_glob_dependent(actual, expected):
    assert is_dict(actual)

    if "followSymlinks" in expected:
        assert is_bool(actual["followSymlinks"], expected["followSymlinks"])

    if "listDirectories" in expected:
        assert is_bool(actual["listDirectories"], expected["listDirectories"])

    if "recurse" in expected:
        assert is_bool(actual["recurse"], expected["recurse"])

    if "relative" in expected:
        assert matches(actual["relative"], expected["relative"])

    check_list_match(lambda a, e: matches(a, e), actual["paths"], expected["paths"], allow_extra=True)

    assert sorted(actual.keys()) == sorted(expected.keys())

def check_object_cmakeFiles(o):
    assert sorted(o.keys()) == ["globsDependent", "inputs", "kind", "paths", "version"]
    # The "kind" and "version" members are handled by check_index_object.
    assert is_dict(o["paths"])
    assert sorted(o["paths"].keys()) == ["build", "source"]
    assert matches(o["paths"]["build"], "^.*/Tests/RunCMake/FileAPI/cmakeFiles-v1-build$")
    assert matches(o["paths"]["source"], "^.*/Tests/RunCMake/FileAPI$")

    expected = [
        {
            "path": "^CMakeLists\\.txt$",
            "isGenerated": None,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^cmakeFiles-v1\\.cmake$",
            "isGenerated": None,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^dir/CMakeLists\\.txt$",
            "isGenerated": None,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^dir/dir/CMakeLists\\.txt$",
            "isGenerated": None,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^dir/dirtest\\.cmake$",
            "isGenerated": None,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^.*/Tests/RunCMake/FileAPIDummyFile\\.cmake$",
            "isGenerated": None,
            "isExternal": True,
            "isCMake": None,
        },
        {
            "path": "^.*/Tests/RunCMake/FileAPI/cmakeFiles-v1-build/generated\\.cmake",
            "isGenerated": True,
            "isExternal": None,
            "isCMake": None,
        },
        {
            "path": "^.*/Modules/CMakeParseArguments\\.cmake$",
            "isGenerated": None,
            "isExternal": True,
            "isCMake": True,
        },
    ]

    expected_globs = [
        {
            "expression": "^.*/Tests/RunCMake/FileAPI/dir/\\*$",
            "paths": [
                "^.*/Tests/RunCMake/FileAPI/dir/dir$",
                "^.*/Tests/RunCMake/FileAPI/dir/dirtest\\.cmake$"
            ],
            "listDirectories": True,
        },
        {
            "expression": "^.*/Tests/RunCMake/FileAPI/dir/\\*\\.cmake$",
            "paths": [
                "^dir/dirtest\\.cmake$"
            ],
            "followSymlinks": True,
            "recurse": True,
            "relative": "^.*/Tests/RunCMake/FileAPI$"
        }
    ]

    inSource = os.path.dirname(o["paths"]["build"]) == o["paths"]["source"]
    if inSource:
        for e in expected:
            e["path"] = e["path"].replace("^.*/Tests/RunCMake/FileAPI/", "^", 1)

    check_list_match(lambda a, e: matches(a["path"], e["path"]), o["inputs"], expected, check=check_input, allow_extra=True)
    check_list_match(lambda a, e: matches(a["expression"], e["expression"]), o["globsDependent"], expected_globs, check=check_glob_dependent, allow_extra=True)

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_objects(index["objects"])



import sys
import json
import re

def is_bool(x):
    return isinstance(x, bool)

def is_dict(x):
    return isinstance(x, dict)

def is_list(x):
    return isinstance(x, list)

def is_int(x):
    return isinstance(x, int) or isinstance(x, long)

def is_string(x):
    return isinstance(x, str) or isinstance(x, unicode)

def check_re(x, regex):
    assert re.search(regex, x)

with open(sys.argv[1]) as f:
    ctest_json = json.load(f)



#!/usr/bin/env python3

import json
import os
import sys

if sys.version_info[0] >= 3:
    unicode = str

trace_file = None
expand = False

for i in sys.argv[1:]:
    if trace_file is None and not i.startswith('-'):
        trace_file = i
        continue

    if i in ['-e', '--expand']:
        expand = True

assert trace_file is not None
assert os.path.exists(trace_file)

if expand:
    msg_args = ['STATUS', 'fff', 'fff;sss;  SPACES !!!  ', ' 42  space in string!', '  SPACES !!!  ']
else:
    msg_args = ['STATUS', 'fff', '${ASDF}', ' ${FOO} ${BAR}', '  SPACES !!!  ']

required_traces = [
    {
        'args': ['STATUS', 'JSON-V1 str', 'spaces'],
        'cmd': 'message',
        'line': 1,
        'line_end': 5
    },
    {
        'args': ['ASDF', 'fff', 'sss', '  SPACES !!!  '],
        'cmd': 'set',
    },
    {
        'args': ['FOO', '42'],
        'cmd': 'set',
    },
    {
        'args': ['BAR', ' space in string!'],
        'cmd': 'set',
    },
    {
        'args': msg_args,
        'cmd': 'message',
        'frame': 3 if expand else 2,
        'global_frame': 3 if expand else 2
    },
    {
        'args': ['STATUS', 'nested global_frame'],
        'cmd': 'message',
        'frame': 3,
        'global_frame': 6 if expand else 5
    },
    {
        'cmd': 'else',
        'global_frame': 4 if expand else 3,
        'line': 3
    }
]

def assert_fields_look_good(line):
    expected_fields = {'args', 'cmd', 'file', 'frame', 'global_frame','line', 'time'}
    if "line_end" in line:
        assert isinstance(line['line_end'], int)
        assert line['line'] != line['line_end']
        expected_fields.add("line_end")

    assert set(line.keys()) == expected_fields

    assert isinstance(line['args'], list)
    assert isinstance(line['cmd'], unicode)
    assert isinstance(line['file'], unicode)
    assert isinstance(line['frame'], int)
    assert isinstance(line['global_frame'], int)
    assert isinstance(line['line'], int)
    assert isinstance(line['time'], float)


with open(trace_file, 'r') as fp:
    # Check for version (must be the first document)
    vers = json.loads(fp.readline())
    assert sorted(vers.keys()) == ['version']
    assert sorted(vers['version'].keys()) == ['major', 'minor']
    assert vers['version']['major'] == 1
    assert vers['version']['minor'] == 2

    for i in fp.readlines():
        line = json.loads(i)
        assert_fields_look_good(line)
        for j in required_traces:
            # Compare the subset of required keys with line
            subset = {
                k: line[k]
                for k in j
                if k in line
            }
            if subset == j:
                required_traces.remove(j)

assert not required_traces, (
    "The following traces were expected to be part of the "
    "output but weren't", required_traces
)



import json
import jsonschema
import os.path
import sys


with open(sys.argv[1], "r", encoding="utf-8-sig") as f:
    contents = json.load(f)

schema_file = os.path.join(
        os.path.dirname(__file__),
        "..", "..", "..", "Help", "manual", "presets", "schema.json")
with open(schema_file, "r", encoding="utf-8") as f:
    schema = json.load(f)

jsonschema.validate(contents, schema)



from check_index import *
import os

def check_objects(o):
    assert is_list(o)
    assert len(o) == 1
    check_index_object(o[0], "configureLog", 1, 0, check_object_configureLog)

def check_object_configureLog(o):
    assert sorted(o.keys()) == ["eventKindNames", "kind", "path", "version"]
    # The "kind" and "version" members are handled by check_index_object.
    path = o["path"]
    assert matches(path, "^.*/CMakeFiles/CMakeConfigureLog\\.yaml$")
    assert os.path.exists(path)
    eventKindNames = o["eventKindNames"]
    assert is_list(eventKindNames)
    assert sorted(eventKindNames) == ["message-v1", "try_compile-v1", "try_run-v1"]

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_objects(index["objects"])



from http.server import HTTPServer, BaseHTTPRequestHandler
import argparse
import time
import subprocess
import sys
import os
import threading
args = None
outerthread = None

barrier = threading.Barrier(2)

class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        barrier.wait()
        self.send_response(200)
        self.end_headers()
        data = b'D'

        if args.speed_limit:
            slow_deadline = time.time()+args.limit_duration

            while time.time() < slow_deadline:
                self.wfile.write(data)
                if args.speed_limit:
                    time.sleep(1.1)

        data = data * 100
        self.wfile.write(data)
        self.close_connection = True

def runServer(fileName):
    httpd = HTTPServer(('localhost', 0), SimpleHTTPRequestHandler)
    with open(fileName,"w") as f:
        f.write('http://localhost:{}/test'.format(httpd.socket.getsockname()[1]))
    httpd.handle_request()
    os.remove(fileName)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--speed_limit', help='transfer rate limitation', action='store_true',default=False)
    parser.add_argument('--limit_duration', help='duration of the transfer rate limitation',default=1, type=float)
    parser.add_argument('--file', help='file to write the url to connect to')
    parser.add_argument('--subprocess', action='store_true')
    args = parser.parse_args()
    if not args.subprocess:
        subprocess.Popen([sys.executable]+sys.argv+['--subprocess'],stdin=subprocess.DEVNULL, stderr=subprocess.DEVNULL,stdout=subprocess.DEVNULL)
    else:
        serverThread = threading.Thread(target=runServer,args=(args.file,))
        serverThread.daemon = True
        serverThread.start()
        barrier.wait(60)
        serverThread.join(20)
